import "Turbine.UI";

-- returns a default local object
function DefaultLocale()
    local L = {};
    setmetatable(L, {__index=function(L, key) return key; end;});
    return L;
end

-- loads the localization strings for the given package
function ImportLocale(package, locale)
    if (locale == nil) then
        locale = Turbine.Engine.GetLocale();
    end
    pcall(import, ("%s.locale.%s"):format(package, locale));
end

-- constrain the window to the display
function ValidatePosition(x, y, width, height)
    x = math.min(math.max(x, 0), Turbine.UI.Display:GetWidth() - width);
    y = math.min(math.max(y, 0), Turbine.UI.Display:GetHeight() - height);
    return x, y;
end

-- deep copy an object
function DeepCopy(object)
    local lookup_table = {}
    local function _copy(object)
        if type(object) ~= "table" then
            return object
        elseif lookup_table[object] then
            return lookup_table[object]
        end
        local new_table = {}
        lookup_table[object] = new_table
        for index, value in pairs(object) do
            new_table[_copy(index)] = _copy(value)
        end
        return setmetatable(new_table, getmetatable(object))
    end
    return _copy(object)
end

-- safely add a listener without overwriting any existing ones
function AddListener(object, event, listener)
    if (object[event] == nil) then
        object[event] = listener;
    else
        if (type(object[event]) == "table") then
            table.insert(object[event], listener);
        else
            object[event] = {object[event], listener};
        end
    end
end

-- safetly remove a listener without clobbering any extras
function RemoveListener(object, event, listener)
    if (object[event] == listener) then
        object[event] = nil;
    else
        if (type(object[event]) == "table") then
            local size = table.getn(object[event]);
            for i = 1, size do
                if (object[event][i] == listener) then
                    table.remove(object[event], i);
                    break;
                end
            end
            -- collapse it back down if we only have one left
            if (table.getn(object[event]) == 1) then
                object[event] = object[event][1];
            end
        end
    end
end

-- safetly execute a listener whether it be an array of functions or a single one
function ExecuteListener(object, event, args)
    if (type(object[event]) == "function") then
        object[event](args);
    else
        if (type(object[event]) == "table") then
            local size = table.getn(object[event]);
            for i = 1, size do
                if (type(object[event][i]) == "function") then
                    object[event][i](args);
                end
            end
        end
    end
end

-- common time formatters, values passed are in seconds
TimeFormat = {};
TimeFormat.LITERAL = function(value)
    return value;
end
TimeFormat.TRUNCATED = function(value)
    if (value >= 3600) then
        if value % 3600 == 0 then
            value = ("%dh"):format(value / 3600)
        else
            value = ("%.1fh"):format(value / 3600)
        end
    elseif (value >= 60) then
        if value % 60 == 0 then
            value = ("%dm"):format(value / 60)
        else
            value = ("%.1fm"):format(value / 60)
        end
    else
        value = ("%ds"):format(value);
    end
    return value;
end
TimeFormat.CLOCK = function(value)
    if (value >= 3600) then
        local sec = math.fmod(value, 3600) / 60;
        value = ("%d:%02d:%02d"):format(value / 3600, value % 3600 / 60, value % 60);
    elseif (value >= 60) then
        value = ("%d:%02d"):format(value / 60, value % 60);
    else
        value = ("0:%02d"):format(value);
    end
    return value;
end
